/*-
 * Copyright (c) 2010 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "codecs.h"

/* do the action in op for a file - or memory */
static int
dofile(codecfun_t *codecfun, const char *f, const char *op, const int fd)
{
	FILE	*codecfp;
	FILE	*fp;
	char	 buf[BUFSIZ * 10];
	int	 passthrough;
	int	 len;
	int	 cc;
	int	 bc;
	int	 rc;
	int	 wc;

	cc = 0;
	len = 0;
	if (!codecs_valid_op(&codecfun->codecs, op)) {
		(void) fprintf(stderr, "codecs: invalid operation '%s'\n", op);
		return 0;
	}
	if (codecs_input_needed(&codecfun->codecs, op)) {
		if (f == NULL) {
			fp = stdin;
		} else if ((fp = fopen(f, "r")) == NULL) {
			(void) fprintf(stderr, "can't open '%s' for reading\n", f);
			return 0;
		}
	} else {
		if (f == NULL || (len = strtol(f, NULL, 10)) <= 0) {
			return 0;
		}
		fp = stdin;
	}
	/* use funopen(3) to perform the transform */
	if ((codecfp = codecs_funopen(codecfun, fp, op)) == NULL) {
		(void) fprintf(stderr, "funopen '%s' failed\n", op);
		return 0;
	}
	/* if we have any input, then write it into the transform */
	if (!codecs_input_needed(&codecfun->codecs, op)) {
		/* this sets the input size internally in codecfp */
		(void) fseek(codecfp, len, SEEK_END);
		(void) fseek(codecfp, CODECS_TRANSFORM, SEEK_END);
		(void) fseek(codecfp, CODECS_SETDRAIN, SEEK_END);
	}
	passthrough = codecs_passthrough(&codecfun->codecs, op);
	for (cc = 0 ; (rc = fread(buf, 1, sizeof(buf) / 2, codecfp)) > 0 ; cc += rc) {
		if (!passthrough) {
			for (bc = 0 ; (wc = write(fd, &buf[bc], rc - bc)) > 0 ; bc += wc) {
			}
		}
	}
	if (passthrough) {
		for (cc = 0 ; (wc = write(fd, &codecfun->output[cc], codecfun->outc - cc)) > 0 ; cc += wc) {
		}
	}
	fclose(codecfp);
	return cc > 0;
}

int
main(int argc, char **argv)
{
	codecfun_t	 codecfun;
	const char	*out;
	FILE		*fp;
	int	 	 fd;
	int	 	 ex;
	int	 	 i;

	(void) memset(&codecfun, 0x0, sizeof(codecfun));
	out = NULL;
	while ((i = getopt(argc, argv, "o:")) != -1) {
		switch(i) {
		case 'o':
			out = optarg;
			break;
		default:
			break;
		}
	}
	ex = EXIT_SUCCESS;
	if (out == NULL) {
		fd = STDOUT_FILENO;
		fp = NULL; /* sigh, only here to shut up the compiler */
	} else {
		if ((fp = fopen(out, "w")) == NULL) {
			(void) fprintf(stderr, "codecs: can't open '%s'\n", out);
			return -1;
		}
		fd = fileno(fp);
	}
	if (optind == argc - 1) {
		if (!dofile(&codecfun, NULL, argv[optind], fd)) {
			ex = EXIT_FAILURE;
		}
	} else {
		for (i = optind + 1 ; i < argc ; i++) {
			if (!dofile(&codecfun, argv[i], argv[optind], fd)) {
				ex = EXIT_FAILURE;
			}
		}
	}
	exit(ex);
}
